import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.Rectangle2D;
import java.util.Random;

import javax.swing.*;

import com.mindfusion.common.DateTime;
import com.mindfusion.common.Duration;
import com.mindfusion.diagramming.*;
import com.mindfusion.diagramming.Shape;


public class MainFrame extends JFrame
{
	DiagramView diagramView;

	JTextField nodesField;
	JTextField linksField;
	JCheckBox routeLinksCheck;
	JLabel elapsedTimeLabel;

	void generateDiagram()
	{
		DateTime start = DateTime.now();

		Random r = new Random();

		// adding items is slower when the diagram is shown inside a view
		// so we create a new diagram here
		Diagram diagram = new Diagram();
		diagram.setDefaultShape(Shape.fromId("Rectangle"));
		diagram.setLinkHeadShape(ArrowHeads.None);

		// adding a lot of items is faster when these properties are set to false
		diagram.setValidityChecks(false);
		diagram.setSelectAfterCreate(false);

		// use QuickRouter for fastest built-in routing
		QuickRouter router = new QuickRouter(diagram);
		router.setGranularity(Granularity.CoarseGrained);
		diagram.setLinkRouter(router);

		int nodes = 5000;
		int links = 5000;
		try
		{
			nodes = Integer.parseInt(nodesField.getText());
			links = Integer.parseInt(linksField.getText());
		}
		catch (Exception ex)
		{
		}

		if (nodes <= 0)
			return;

		int size = (int)Math.sqrt(nodes) * 60;

        diagram.setEnableSpatialIndex(nodes + links > 4000);

		for (int i = 0; i < nodes; ++i)
		{
			diagram.getFactory().createShapeNode(
				r.nextInt(size), r.nextInt(size), 20, 10);
		}

		for (int i = 0; i < links; ++i)
		{
			diagram.getFactory().createDiagramLink(
				diagram.getNodes().get(r.nextInt(nodes - 1)),
				diagram.getNodes().get(r.nextInt(nodes - 1)));
		}

		if (routeLinksCheck.isSelected())
			diagram.routeAllLinks();

		Duration elapsed = DateTime.now().subtract(start);
		elapsedTimeLabel.setText(elapsed.toString());

		// finally show the diagram inside the view
		diagramView.setDiagram(diagram);
		diagram.setBounds(
			new Rectangle2D.Float(0, 0, size + 20, size + 20));

		diagramView.bringIntoView(
			diagram.getNodes().first());
	}

    private void initFrame()
    {
    	setTitle("MindFusion.Diagramming Sample: Stress Test");
        setSize(new Dimension(1200, 900));
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        setLocationByPlatform(true);

    	diagramView = new DiagramView();

    	JScrollPane scrollPane = new JScrollPane(diagramView);

        JButton generateButton = new JButton("Generate");
        generateButton.addActionListener(new ActionListener() {
			@Override public void actionPerformed(ActionEvent e) {
				generateDiagram();
			}
		});

        routeLinksCheck = new JCheckBox("Route links");

        JLabel nodesLabel = new JLabel("Nodes:");
        nodesField = new JTextField("5000");
        nodesField.setMaximumSize(
            new Dimension(
            	Integer.MAX_VALUE,
            	nodesField.getPreferredSize().height));

        JLabel linksLabel = new JLabel("Links:");
        linksField = new JTextField("5000");
        linksField.setMaximumSize(
            new Dimension(
            	Integer.MAX_VALUE,
            	linksField.getPreferredSize().height));

        elapsedTimeLabel = new JLabel();

        JPanel spacer = new JPanel();
        spacer.setPreferredSize(new Dimension(200, 200));

    	JTextArea descriptionArea = new JTextArea();
        descriptionArea.setBackground(Color.white);
        descriptionArea.setName("label1");
        descriptionArea.setText(
        	"This example shows the fastest way to add a lot of items to the diagram. It also " +
        	"optionally runs the link routing functions to find routes between the diagram nodes.");
        descriptionArea.setEditable(false);
        descriptionArea.setAutoscrolls(false);
        descriptionArea.setLineWrap(true);
        descriptionArea.setWrapStyleWord(true);

		JPanel controlPanel = new JPanel();
		controlPanel.setLayout(
			new BoxLayout(controlPanel, BoxLayout.Y_AXIS));
		controlPanel.add(generateButton);
		controlPanel.add(routeLinksCheck);
		controlPanel.add(nodesLabel);
		controlPanel.add(nodesField);
		controlPanel.add(linksLabel);
		controlPanel.add(linksField);
		controlPanel.add(elapsedTimeLabel);
		controlPanel.add(spacer);
		controlPanel.add(descriptionArea);

		for (Component child : controlPanel.getComponents())
		{
			if (child instanceof JComponent)
			{
				JComponent jchild = (JComponent)child;
				jchild.setAlignmentX(LEFT_ALIGNMENT);
			}
		}

        Container cp = this.getContentPane();
      	cp.setLayout(new BorderLayout());
      	cp.add(scrollPane, BorderLayout.CENTER);
		cp.add(controlPanel, BorderLayout.EAST);
    }

	public MainFrame()
	{
        initFrame();
    }

	private static final long serialVersionUID = 1L;
}
